import asyncio
import json
import ssl
import time
import uuid

import certifi
import streamlit as st
import streamlit.components.v1 as components
import websockets


def getOrCreateUID():
    if 'uid' not in st.session_state:
        st.session_state['uid'] = ''
    st.session_state['uid'] = st.session_state['uid'] or str(uuid.uuid1())
    print ('getOrCreateUID: ', st.session_state['uid'])
    return st.session_state['uid']


# Generate a unique uid that gets embedded in components.html for frontend
# Both frontend and server connect to ws using the same uid
# server sends commands like localStorage_get_key, localStorage_set_key, localStorage_clear_key etc. to the WS server,
# which relays the commands to the other connected endpoint (the frontend), and back
def injectWebsocketCode(hostPort, uid):
    code = '<script>function connect() { console.log("in connect uid: ", "' + uid + '"); var ws = new WebSocket("wss://' + hostPort + '/?uid=' + uid + '");' + """
  ws.onopen = function() {
    // subscribe to some channels
    // ws.send(JSON.stringify({ status: 'connected' }));
    console.log("onopen");
  };

  ws.onmessage = function(e) {
    console.log('Message:', e.data);
    var obj = JSON.parse(e.data);
    if (obj.cmd == 'localStorage_get_key') {
        var val = localStorage[obj.key] || '';
        ws.send(JSON.stringify({ status: 'success', val }));
        console.log('returning: ', val);
    } else if (obj.cmd == 'localStorage_set_key') {
        localStorage[obj.key] = obj.val;
        ws.send(JSON.stringify({ status: 'success' }));
        console.log('set: ', obj.key, obj.val);
    }
  };

  ws.onclose = function(e) {
    console.log('Socket is closed. Reconnect will be attempted in 1 second.', e.reason);
    setTimeout(function() {
      connect();
    }, 1000);
  };

  ws.onerror = function(err) {
    console.error('Socket encountered error: ', err.message, 'Closing socket');
    ws.close();
  };
}

connect();
</script>
        """
    components.html(code, height=0)
    time.sleep(1)       # Without sleep there are problems
    return WebsocketClient(hostPort, uid)


class WebsocketClient:
    def __init__(self, hostPort, uid):
        self.hostPort = hostPort
        self.uid = uid
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

    def sendCommand(self, value, waitForResponse=True):
        ssl_context = ssl.create_default_context()
        ssl_context.load_verify_locations(certifi.where())

        async def query(future):
            async with websockets.connect("wss://" + self.hostPort + "/?uid=" + self.uid, ssl=ssl_context) as ws:
                await ws.send(value)
                if waitForResponse:
                    response = await ws.recv()
                    print('response: ', response)
                    future.set_result(response)
                else:
                    future.set_result('')

        future1 = asyncio.Future()
        self.loop.run_until_complete(query(future1))
        print('future1.result: ', future1.result())
        return future1.result()

    def getLocalStorageVal(self, key):
        result = self.sendCommand(json.dumps({ 'cmd': 'localStorage_get_key', 'key': key }))
        return json.loads(result)['val']

    def setLocalStorageVal(self, key, val):
        result = self.sendCommand(json.dumps({ 'cmd': 'localStorage_set_key', 'key': key, 'val': val }))
        return result




import json
from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import urlparse, parse_qs
import logging

from streamlit_ws_localstorage import WebsocketClient

hostName = "0.0.0.0"
serverPort = 8002


# Server to handle /redirect urls that come from OAuth 2 redirects
# Upon receiving the code and state variables from the request, sends it to the Websocket server with the state as uid
# The auth initiator should be listening to the websocket server using same the uid, thus it can receive the code
class MyServer(BaseHTTPRequestHandler):
    def do_GET(self):
        try:
            parsed_url = urlparse(self.path)
            d = parse_qs(parsed_url.query)
            if parsed_url.path == '/redirect' and 'code' in d and 'state' in d:
                # Validation passed
                code = d['code'][0]
                state = d['state'][0]
                logging.info('code, state: {} {}'.format(code, state))
                conn = WebsocketClient('linode.liquidco.in', state)
                ret = conn.sendCommand(json.dumps({ 'cmd': 'process_auth_redirect', 'code': code, 'state': state }), waitForResponse=False)
                ret = 'AUTH SUCCESS ' + ret
                self.sendText(ret, closeTab=True)
            else:
                # Validation failed
                self.sendText('AUTH FAILED: Bad url', status=400)
        except Exception:
            self.sendText('AUTH FAILED: Exception', status=502)
            logging.exception("Exception")

    def sendText(self, text, status=200, closeTab=False):
        self.send_response(status)
        self.send_header("Content-type", "text/html")
        self.end_headers()

        script = '<div style="font-size: 14px; margin-top: 10px;">This tab will automatically close soon. If it doesnt, please close it manually.</div><script>setTimeout(() => window.close(), 2000);</script>' if closeTab else ''
        html = '<html><head><title>Auth redirect server</title></head><body><div style="font-size: 20px; margin-top: 10px;">{}</div>{}</body></html>'.format(text, script)
        self.wfile.write(bytes(html, "utf-8"))


if __name__ == "__main__":
    webServer = HTTPServer((hostName, serverPort), MyServer)
    print("Server started http://%s:%s" % (hostName, serverPort))

    try:
        webServer.serve_forever()
    except KeyboardInterrupt:
        pass

    webServer.server_close()
    print("Server stopped.")



import time

import streamlit.components.v1 as components


def loginWithOAuthComponent(hostPort, uid, localStorageCodeKey, reloadInSecs=5, height=40):
    code = """<div id="statusDiv">Waiting for auth code</div>
<script>
function connect(hostPort, uid, localStorageCodeKey, reloadInSecs) {
  console.log("in loginWithOAuthComponent connect uid: ", uid);
  var ws = new WebSocket("wss://" + hostPort + "/?uid=" + uid);
  ws.onopen = function() {
    // subscribe to some channels
    console.log("loginWithOAuthComponent onopen");
  };

  ws.onmessage = function(e) {
    console.log('loginWithOAuthComponent onmessage:', e.data);
    var obj = JSON.parse(e.data);
    if (obj.cmd == 'process_auth_redirect') {
        console.log('process_auth_redirect: ', obj);
        var code = obj['code'];
        localStorage[localStorageCodeKey] = code;
        console.log('saving code: ', code);

        var div = document.getElementById("statusDiv");
        div.innerHTML = 'Auth code received. Reloading page in ' + reloadInSecs + ' seconds';
        setTimeout(() => window.parent.location.reload(), reloadInSecs*1000);
    }
  };

  ws.onclose = function(e) {
    console.log('loginWithOAuthComponent onclose: Socket is closed. Reconnect will be attempted in 1 second.', e.reason);
    setTimeout(function() {
      connect();
    }, 1000);
  };

  ws.onerror = function(err) {
    console.error('loginWithOAuthComponent onerror: Socket encountered error: ', err.message, 'Closing socket');
    ws.close();
  };
}""" + "connect('{}', '{}', '{}', {});</script>".format(hostPort, uid, localStorageCodeKey, reloadInSecs)
    components.html(code, height=height)
    time.sleep(1)



import asyncio
import ssl

import certifi
import websockets


def main():
    ssl_context = ssl.create_default_context()
    ssl_context.load_verify_locations(certifi.where())

    async def query(future):
        async with websockets.connect("wss://linode.liquidco.in/?uid=21", ssl=ssl_context) as ws:
            await ws.send('{"cmd":"echo","msg":"hi4"}')
            response = await ws.recv()
            print ('response: ', response)
            future.set_result(response)

    future1 = asyncio.Future()
    asyncio.run(query(future1))
    print ('future1.result: ', future1.result())


main()



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='streamlit_ws_localstorage',
    packages=setuptools.find_packages(),
    version='1.0.5',  # Ideally should be same as your GitHub release tag varsion
    description="A simple synchronous way of accessing localStorage from your Streamlit app.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Gagandeep Singh',
    author_email='gagan@heloprotocol.in',
    url='https://github.com/gagangoku/streamlit-ws-localstorage',
    download_url='https://github.com/gagangoku/streamlit-ws-localstorage/archive/refs/tags/1.0.5.tar.gz',
    keywords=['python', 'websocket', 'cookies', 'localstorage', 'streamlit'],
    classifiers=[],
    python_requires=">=3.6",
    install_requires=[
        "streamlit >= 0.86",
    ],
)



import json
from urllib.parse import urlparse, parse_qs

from simple_websocket_server import WebSocketServer, WebSocket


# Generate a unique uid that gets embedded in components.html for frontend
# Both frontend and server connect to ws using the same uid
# server sends commands like localStorage_get_key, localStorage_set_key, localStorage_clear_key etc.
class SimpleChat(WebSocket):
    uid: str = None

    def handle(self):
        print ('handle: ', self.uid, self.data, self.address)

        # Echo back
        try:
            obj = json.loads(self.data)
            if 'cmd' in obj and obj['cmd'] == 'echo':
                self.send_message(self.data)
        except:
            print ('exception in handle, ignoring')

        for client in clients[self.uid]:
            if client != self:
                client.send_message(self.data)

    def connected(self):
        print('connected: ', self.address, self.request.path)
        self.uid = self.request.path
        try:
            parsed_url = urlparse(self.request.path)
            d = parse_qs(parsed_url.query)
            if 'uid' in d and len(d['uid']) > 0:
                self.uid = d['uid'][0]
            else:
                self.send_message('No uid found')
                self.close(1, 'No uid found')
                return
        except:
            self.send_message('Exception')
            self.close(2, 'Exception')
            return

        if self.uid not in clients:
            clients[self.uid] = []
        clients[self.uid].append(self)

    def handle_close(self):
        print ('handle_close: ', self.uid, self.address)
        clients[self.uid].remove(self)
        print(self.address, 'closed')


clients = dict()

server = WebSocketServer('0.0.0.0', 8001, SimpleChat)
server.serve_forever()
